Attribute VB_Name = "basRndData"
Option Explicit

' ***************************************************************************
' Project:
'
' Module:        basRndData
'
' Description:   Build a random generated data string using my unique
'                gathering methods.  I now have three different ways to
'                create a new seed for the random generator.
'
'                      Seed1_Random_Generator
'                      Seed2_Random_Generator
'                      Seed3_Random_Generator
'
'                These are referenced in the Build_Random_Data and
'                Non_Repeating_Numbers routines.
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Module created by kenaso@home.com
' 11-FEB-2000  Kenneth Ives     Increased the Window and Child handle
'                               accumulators from long integer to double to
'                               accomodate Windows NT.  Thanks to Charles
'                               Zimmerman for catching this.
' ***************************************************************************

' ------------------------------------------------------------
' This is a rough translation of the GetTickCount API.  The
' tick count of a PC is only valid for the first 49.7 days
' since it was last rebooted.  When you capture the tick
' count, you are capturing the total number of milliseconds
' since the PC was last rebooted.
' ------------------------------------------------------------
  Private Declare Function GetTickCount Lib "kernel32" () As Long
  
' ------------------------------------------------------------
' APIs for building strings in memory
' By Jody Gelowitz    http://www.visual-statement.com/vb
' ------------------------------------------------------------
  Private Declare Function VarPtrArray Lib "msvbvm60.dll" Alias "VarPtr" _
          (Ptr() As Any) As Long
           
  Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
          (pDst As Any, pSrc As Any, ByVal ByteLen As Long)

' ------------------------------------------------------------
' API function for obtaining volume information
' ------------------------------------------------------------
  Private Declare Function GetVolumeInformation Lib "kernel32" Alias "GetVolumeInformationA" _
          (ByVal lpRootPathName As String, ByVal lpVolumeNameBuffer As String, _
           ByVal nVolumeNameSize As Long, lpVolumeSerialNumber As Long, _
           lpMaximumComponentLength As Long, lpFileSystemFlags As Long, _
           ByVal lpFileSystemNameBuffer As String, ByVal nFileSystemNameSize As Long) As Long

' ------------------------------------------------------------
' This ENumWindows call is one of the most useful.
' ------------------------------------------------------------
  Private Declare Function EnumWindows Lib "user32" _
          (ByVal lpfn As Long, ByVal lParam As Long) As Boolean
  
  Private Declare Function EnumChildWindows Lib "user32" _
          (ByVal hWndParent As Long, ByVal lpEnumFunc As Long, _
           ByVal lParam As Long) As Long
  
' ------------------------------------------------------------
' Define local variables.
' "m_" denotes module level variable.
' ------------------------------------------------------------
  Private m_dblHandleTotal As Double
  Private m_dblParentHandle() As Double
  
Public Function Use_Hex_Chars(strValue As String) As String

' ***************************************************************************
' Routine:       Use_Hex_Chars
'
' Description:   This routine will convert an ASCII character to its
'                2-character hex representation.
'
' Parameters:    strValue - input string of ASCII character
'
' Return Values: String of hexidecimal characters
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Define variables
' ------------------------------------------------------------
  Dim n As Long
  Dim lngPosition As Long
  Dim lngDataLength As Long
  Dim intChar As Integer
  Dim strChar As String
  Dim strSource As String
  Dim strOutput As String
  
' ------------------------------------------------------------
' If printable characters are to be used, then reinitialize
' the variables.
' ------------------------------------------------------------
  strSource = ""
  lngPosition = 0
  lngDataLength = Len(strValue)
  strOutput = Space(10240)
  
' ------------------------------------------------------------
' If Hex characters are to be used, then convert each decimal
' value to its 2-char hex representation.
' -------------------------------------------------------------
  For n = 1 To lngDataLength
  
      intChar = Asc(Mid(strValue, n, 1))  ' capture one char at a time
      strChar = Hex(intChar)                 ' convert ASCII decimal to hex
          
      ' if only one char is returned
      ' then prefix with a leading zero
      If Len(strChar) < 2 Then
          strChar = "0" & strChar
      End If
      
      ' Append to output string
      strSource = strChar
      
      ' allocate additional space if we need it
      If lngPosition + LenB(strSource) > LenB(strOutput) Then
          strOutput = strOutput & Space(10240)
      End If
        
      ' append the data in memory and keep track
      ' of the space allocations
      CopyMemory ByVal StrPtr(strOutput) + lngPosition, ByVal StrPtr(strSource), LenB(strSource)
      lngPosition = lngPosition + LenB(strSource)
  Next
  
' -------------------------------------------------------------
' Adjust the size of the data string before returning it
' -------------------------------------------------------------
  strOutput = Left(strOutput, lngPosition / 2)
  
' -------------------------------------------------------------
' Return the completed string
' -------------------------------------------------------------
  Use_Hex_Chars = strOutput

' -------------------------------------------------------------
' Empty variables for security reasons
' -------------------------------------------------------------
  strOutput = ""
  strSource = ""
  lngPosition = 0
  
End Function

Public Function Create_Char_Data(intType As Integer, _
                                 Optional lngDataLength As Long = 50) As String

' ***************************************************************************
' Routine:       Create_Char_Data
'
' Description:   This routine will build a string of random generated data
'                based on type of data desired and the length of the string.

'
' Parameters:    intType - 1 = Printable keyboard characters only (33 to 126)
'                        2 = All ASCII characters (0 to 255)
'                lngDataLength - length of the data string
'
' Return Values: STring of random generated data
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' -------------------------------------------------------------
' Define local variables
' -------------------------------------------------------------
  Dim n As Long
  Dim lngPosition As Long
  Dim intChar As Integer
  Dim strSource As String
  Dim strOutput As String

' ------------------------------------------------------------
' Define local variables
' ------------------------------------------------------------
  lngPosition = 0
  strSource = ""
  strOutput = Space(10240)
  
' ------------------------------------------------------------
' Build a string of randomly generated characters using 1k
' chunks.  The output string can be longer if desired.
' ------------------------------------------------------------
  For n = 1 To lngDataLength
  
      Select Case intType
             ' Printable keyboard characters
             Case 1:
                  ' Loop until we get a printable character
                  ' value of 33 to 126 (0x21 to 0x7E)
                  Do
                      intChar = Int(Rnd * 126)    ' generate random integer 0 to 126
                  Loop Until (intChar > 32 And intChar < 127)
                  
             ' Use all ASCII characters (0-255)
             Case 2:
                  ' generate integer value 0 to 255
                  intChar = Int(Rnd * 255)
      End Select
      
      ' convert decimal value to ASCII character
      strSource = Chr(intChar)
      
      ' allocate additional space if we need it
      If lngPosition + LenB(strSource) > LenB(strOutput) Then
          strOutput = strOutput & Space(10240)
      End If
        
      ' append the data in memory and keep track
      ' of the space allocations
      CopyMemory ByVal StrPtr(strOutput) + lngPosition, ByVal StrPtr(strSource), LenB(strSource)
      lngPosition = lngPosition + LenB(strSource)
  Next
  
' ------------------------------------------------------------
' Adjust the size of the data string before returning it
' ------------------------------------------------------------
  strOutput = Left(strOutput, lngPosition / 2)
      
Normal_Exit:
' ------------------------------------------------------------
' Return the completed string
' ------------------------------------------------------------
  Create_Char_Data = strOutput
  
' ------------------------------------------------------------
' Empty variables for security reasons
' ------------------------------------------------------------
  strSource = ""
  strOutput = ""
  lngPosition = 0
  
End Function

Private Function Reshuffle_The_Data(strInputText As String, _
                                    bUse_Keyboard_Chars As Boolean) As String

' ***************************************************************************
' Routine:       Reshuffle_The_Data
'
' Description:   This function will accept an incoming string and will
'                perform an XoR binary compare on each byte to create a new
'                ASCII decimal value.
'
' Parameters:    bUse_Keyboard_Chars - TRUE or FALSE
'
' Return Values: New string of data
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Define variables
' ------------------------------------------------------------
  Dim lngStrLength As Long
  Dim n As Long
  Dim lngPosition As Long
  
  Dim intOrigChar As Integer
  Dim intNewChar As Integer
  Dim intXorChar As Integer
  
  Dim strSource As String
  Dim strOutput As String
  Dim strTmp As String

' ------------------------------------------------------------
' Initialize local variables
' ------------------------------------------------------------
  lngPosition = 0
  strSource = ""
  strOutput = Space(10240)
  lngStrLength = Len(strInputText)
  strTmp = strInputText
  
' ------------------------------------------------------------
' See if anything was passed to be processed
' ------------------------------------------------------------
  If lngStrLength = 0 Then
      Reshuffle_The_Data = ""
      Exit Function
  End If

' ------------------------------------------------------------
' Parse the incoming string and process each byte
' ------------------------------------------------------------
  For n = 1 To lngStrLength
  
      ' convert each byte to its decimal value
      intOrigChar = Asc(Mid(strTmp, n, 1))
      
      ' All addition is performed modulo 256. This means
      ' that when you add two values, the result cannot
      ' exceed 255 (that's 256 possible values, including
      ' zero). The result cannot exceed 255 because we only
      ' use bytes. Therefore, if the result of the
      ' addition would normally exceed 255, we wrap it around
      ' and start with zero. For example:
      '
      ' normal addition:        257 = 250 + 7  <- Beyond the 255 range
      ' addition modulo 256:    1 = (250 + 7) MOD 256
      '
      intNewChar = (Int(Rnd * 255) + 17) Mod 256

      ' Use the XoR operator to perform logical exclusion on two
      ' expressions.  The XoR operator performs as both a logical
      ' and bitwise operator.  A bit-wise comparison of two
      ' expressions using exclusive-or logic to form the result,
      ' as shown in the following example:
      '
      '  x = 1000 XoR 1110
      '
      '      1000
      '      1110
      '     ------
      '  x = 0110   Same values become "0" else become "1"
      '
      intXorChar = intOrigChar Xor intNewChar

      ' If we are only using printable characters
      If bUse_Keyboard_Chars Then
          
          Do
              ' if less than ASCII 33 then add 33
              If intXorChar < 33 Then
                  intXorChar = intXorChar + 33
              End If
            
              ' if greater than ASCII 126 then add 130
              If intXorChar > 126 Then
                  intXorChar = (intXorChar + 130) Mod 256
              End If
              
          Loop Until intXorChar > 32 And intXorChar < 127
      End If
      
      ' append the new character to the temporary output string
      strSource = Chr(intXorChar)
      
      ' allocate additional space if we need it
      If lngPosition + LenB(strSource) > LenB(strOutput) Then
          strOutput = strOutput & Space(10240)
      End If
        
      ' append the data in memory and keep track
      ' of the space allocations
      CopyMemory ByVal StrPtr(strOutput) + lngPosition, ByVal StrPtr(strSource), LenB(strSource)
      lngPosition = lngPosition + LenB(strSource)
  Next
  
' ------------------------------------------------------------
' Adjust the size of the data string before returning it
' ------------------------------------------------------------
  strOutput = Left(strOutput, lngPosition / 2)
  
' ------------------------------------------------------------
' Return the reshuffled data
' ------------------------------------------------------------
  Reshuffle_The_Data = strOutput

' ------------------------------------------------------------
' Empty variables for security reasons
' ------------------------------------------------------------
  lngStrLength = 0
  lngPosition = 0
  strSource = ""
  strOutput = ""
  strTmp = ""
  
End Function
Private Function EnumChildWindowsProc(ByVal lngChildHandle As Long, lngParam As Long) As Long
  
' ***************************************************************************
' Routine:       EnumChildWindowsProc
'
' Description:   Gather all of the active sub window handle values.
'
' Parameters:    lngChildHandle - handle number of the current queried
'                application
'                lngParam - not used
'
' Return Values:
'
' Special Logic: Running total of all sub window handles
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Add the child window handle values to the counter
' ------------------------------------------------------------
  m_dblHandleTotal = m_dblHandleTotal + lngChildHandle
  EnumChildWindowsProc = True

End Function

Private Function EnumWindowsProc(ByVal lngParentHandle As Long, lngParam As Long) As Long
    
' ***************************************************************************
' Routine:       EnumWindowsProc
'
' Description:   Gather all of the active sub window handle values.
'
' Parameters:    lngParentHandle - handle number of the current queried
'                          application
'                lngParam - not used
'
' Return Values:
'
' Special Logic: Running total of all active window handles
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Add the parent window handle values to the array
' ------------------------------------------------------------
  ReDim Preserve m_dblParentHandle(UBound(m_dblParentHandle) + 1)
  m_dblParentHandle(UBound(m_dblParentHandle)) = lngParentHandle
  EnumWindowsProc = True

End Function

Public Function Build_Random_Data(lngDataLength As Long, _
                                  bUse_Hex_Chars As Boolean, _
                                  bUse_Keyboard_Chars As Boolean) As String

' ***************************************************************************
' Routine:       Build_Random_Data
'
' Description:   Build a random data string.  Keep it within reason so as
'                not to clog your memory processes.  The maximum value I
'                have ever used was 32768 (1024 * 32).
'
'
' Parameters:    lngDataLength = Length of the data string
'
'                bUse_Hex_Chars      True  - convert each char to its
'                                            2-character hex representation
'                                                Ex:  15  becomes  0F
'                                    False - do not convert
'
'                bUse_Keyboard_Chars True  - use printable values
'                                            (33 to 126)
'                                    False - use all ASCII values
'                                            (0 to 255)
'
' Return Values: String of random generated data
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Define local variables
' ------------------------------------------------------------
  Dim strTmp1 As String
  Dim strTmp2 As String
  Dim strRnd_Data As String
  
' ------------------------------------------------------------
' Define local variables
' ------------------------------------------------------------
  strTmp1 = ""
  strTmp2 = ""
  strRnd_Data = ""
  
' ------------------------------------------------------------
' If length of the desired data string equal zero then leave
' ------------------------------------------------------------
  If lngDataLength = 0 Then Exit Function
  
' ------------------------------------------------------------
' Seed the random generator.  Determine which seed version
' you want to use by uncommenting one of the selections
' below.
' ------------------------------------------------------------
  Seed1_Random_Generator
 ' Seed2_Random_Generator
 ' Seed3_Random_Generator
  
' ------------------------------------------------------------
' Build a string of randomly generated characters
' ------------------------------------------------------------
  If bUse_Keyboard_Chars Then
      ' Use ASCII values 33 to 126.  All keyboard
      ' printable characters
      strTmp1 = Create_Char_Data(1, lngDataLength)
  Else
      ' use all the ASCII characters (0-255)
      strTmp1 = Create_Char_Data(2, lngDataLength)
  End If
  
' ------------------------------------------------------------
' Mix up the random generated data some more
' ------------------------------------------------------------
  strRnd_Data = Reshuffle_The_Data(strTmp1, bUse_Keyboard_Chars)

' ------------------------------------------------------------
' Return final out based on user requirements
' ------------------------------------------------------------
  If bUse_Hex_Chars Then
      Build_Random_Data = Use_Hex_Chars(strRnd_Data)  ' 2-Char hex output
  Else
      Build_Random_Data = strRnd_Data                 ' Decimal value output
  End If
  
' ------------------------------------------------------------
' Empty variables for security purposes
' ------------------------------------------------------------
  strTmp1 = ""
  strTmp2 = ""
  strRnd_Data = ""
  
End Function

Public Sub Seed1_Random_Generator()

' ***************************************************************************
' Routine:       Seed1_Random_Generator
'
' Description:   Generates a 64-bit (8-byte word) double precision number
'                as the new seed for the random generator.  The random
'                generator takes this number and then creates a 32-bit (4-
'                byte) single precision number.  Reseed as often as you
'                want.
'
'                This type of random generation is needed for gaming,
'                password creation, encryption, etc.  A calculated value
'                that cannot be easily compromised.
'
' Parameters:
'
' Return Values:
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Define variables
' ------------------------------------------------------------
  Dim i As Long
  Dim bTestMode As Boolean
  Dim lngParam As Long
  Dim lngRetVal As Long
  Dim lngVolSn As Long
  Dim lngTickCnt As Long
  
  ' 64-bit(8-byte) floating-point number variables
  Dim Pi As Double
  Dim varRndSeed As Variant
  Dim varTmpSeed As Variant
  Dim varDecimal As Variant
  
Top_Of_Test_Loop:
' ------------------------------------------------------------
' Empty all variables for security purposes
' ------------------------------------------------------------
  Erase m_dblParentHandle()           ' Remove all the data from the array
  ReDim m_dblParentHandle(0)          ' Resize array so length is not known
  m_dblHandleTotal = 0                ' reset other variables to zero
  lngVolSn = 0
  lngTickCnt = 0
  varRndSeed = 0
  varTmpSeed = 0
  varDecimal = 0
  bTestMode = False
  
' ------------------------------------------------------------
' Get numeric value of all active windows
' ------------------------------------------------------------
  lngRetVal = EnumWindows(AddressOf EnumWindowsProc, lngParam)
    
' ------------------------------------------------------------
' Accumulate the total values of the active window handles
' ------------------------------------------------------------
  For i = 1 To UBound(m_dblParentHandle)
      m_dblHandleTotal = m_dblHandleTotal + m_dblParentHandle(i)
  Next
    
' ------------------------------------------------------------
' Get numeric value of all active child windows
' ------------------------------------------------------------
  For i = 1 To UBound(m_dblParentHandle)
      lngRetVal = EnumChildWindows(m_dblParentHandle(i), AddressOf EnumChildWindowsProc, 0)
  Next

' ------------------------------------------------------------
' Calculate Pi to 14 decimal places which should be
' 3.14159265358979
' ------------------------------------------------------------
  Pi = Atn(1) * 4
  
' ------------------------------------------------------------
' Use API call to get volume serial number of drive C:.
' Assumes there will always be a drive C: since we are using
' Windows.
' ------------------------------------------------------------
  lngRetVal = GetVolumeInformation("C:\", "", 0, lngVolSn, 0, 0, "", 0)
   
' ------------------------------------------------------------
' get number of milliseconds since the last reboot
' ------------------------------------------------------------
  lngTickCnt = GetTickCount
  
' ------------------------------------------------------------
' add together and convert to double precision
' ------------------------------------------------------------
  varTmpSeed = CDec(lngTickCnt + Timer + Pi + CDbl(Now()))
  
' ------------------------------------------------------------
' Pre-Initialize the VB random generator
' ------------------------------------------------------------
  Randomize varTmpSeed
  
' ------------------------------------------------------------
' Create a new seed value based on the following:
'
'  1.  System data/time stamp in double precision
'  2.  Random generated value from Pi
'  3.  Random generated value the tickcount
'  4.  Accumulated total of all active windows & sub windows
'  5.  Volume serial number of drive C: in long integer
'  6.  Random generated value from the tickcount
'  7.  Random generated value from the volumn serial number
'  8.  Add Pi to the running total
'  9.  Capture just the decimal portion of the running total
' 10.  Add Pi to the decimal value
' 11.  Multiply new decimal value by random value of 1
' 12.  Convert this value using CDec() function of VB
' 13.  Add running total and decimal value together
' 14.  Add running total and tickcount
' ------------------------------------------------------------
  varRndSeed = CDbl(Now()) + (Rnd * Pi)
  varRndSeed = varRndSeed + CDbl((Rnd * lngTickCnt) + m_dblHandleTotal + lngVolSn)
  varRndSeed = varRndSeed + (Rnd * m_dblHandleTotal) + (Rnd * lngVolSn) + Pi
  
  varDecimal = Mid(varRndSeed, InStr(varRndSeed, ".")) ' save decimal value only
  varDecimal = CDec((varDecimal + Pi) * (Rnd * 1))
  
  varRndSeed = varRndSeed + varDecimal + lngTickCnt
  
' ------------------------------------------------------------
' For debugging only.
' To turn on debugging, uncomment the next line to set
' bTestMode = True
'
' Sample Output:    238688296.59674506039113906
'                   260981561.7276916851491254
'                   142932264.06364296356874
'                   209517593.4024467258857
'                   250310710.85324095735983
'                   165968569.997226489607152
' ------------------------------------------------------------
  'bTestMode = True
  If bTestMode Then
      Debug.Print varRndSeed
      Stop                    ' press F5 to continue
      GoTo Top_Of_Test_Loop
  End If
  
' ------------------------------------------------------------
' Initialize the random generator using the new calculated
' value (64-bit value) as the seed.  Randomize returns a
' data type that stores a 32-bit(4-byte) single-precision
' floating-point number.
' ------------------------------------------------------------
  Randomize varRndSeed
  
' ------------------------------------------------------------
' Empty all variables for security purposes
' ------------------------------------------------------------
  Erase m_dblParentHandle()    ' Remove all the data from the array
  ReDim m_dblParentHandle(0)   ' Resize array so length is not known
  m_dblHandleTotal = 0         ' reset other variables to zero
  lngVolSn = 0
  lngTickCnt = 0
  varRndSeed = Empty
  varTmpSeed = Empty
  varDecimal = Empty
  
End Sub

Public Sub Seed2_Random_Generator()

' ***************************************************************************
' Routine:       Seed2_Random_Generator
'
' Description:   Generates a 64-bit (8-byte word) double precision number
'                as the new seed for the random generator.  The random
'                generator takes this number and then creates a 32-bit (4-
'                byte) single precision number.  Reseed as often as you
'                want.
'
'                This type of random generation is needed for gaming,
'                password creation, encryption, etc.  A calculated value
'                that cannot be easily compromised.
'
' Parameters:
'
' Return Values:
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Define local variables
' ------------------------------------------------------------
  Dim sngMultiplier As Single
  Dim n As Long
  Dim lngStart As Long
  Dim lngFinish As Long
  Dim lngTmpHold As Long
  Dim lngTmpValue As Long
  Dim lngParam As Long
  Dim lngRetVal As Long
  Dim lngVolSn As Long
  Dim lngTickCnt As Long
  Dim bTestMode As Boolean
    
  ' 64-bit(8-byte) floating-point number variables
  Dim Pi As Double
  Dim dblTmpValue As Double
  Dim dblCurrTime As Double
  Dim varRndSeed As Variant
  Dim varTmpSeed As Variant
  Dim varDecimal As Variant
  
  
Top_Of_Test_Loop:
' ------------------------------------------------------------
' Initialize variables
' ------------------------------------------------------------
  bTestMode = False
  sngMultiplier = 1.321        ' Pick your own number. I chose 1.321
  Erase m_dblParentHandle()    ' Remove all the data from the array
  ReDim m_dblParentHandle(0)   ' Resize array so length is not known
  varRndSeed = 0
  varDecimal = 0
  varTmpSeed = 0
  dblCurrTime = 0
  dblTmpValue = 0
  m_dblHandleTotal = 0
  lngStart = 0
  lngFinish = 0
  lngTmpValue = 0
  lngTmpHold = 0
  lngVolSn = 0
  lngTickCnt = 0
  lngParam = 0
  
' ------------------------------------------------------------
' Get numeric value of all active windows
' ------------------------------------------------------------
  lngRetVal = EnumWindows(AddressOf EnumWindowsProc, lngParam)
    
' ------------------------------------------------------------
' Accumulate the total values of the active window handles
' ------------------------------------------------------------
  For n = 1 To UBound(m_dblParentHandle)
      m_dblHandleTotal = m_dblHandleTotal + m_dblParentHandle(n)
  Next
    
' ------------------------------------------------------------
' Get numeric value of all active child windows
' ------------------------------------------------------------
  For n = 1 To UBound(m_dblParentHandle)
      lngRetVal = EnumChildWindows(m_dblParentHandle(n), AddressOf EnumChildWindowsProc, 0)
  Next

' ------------------------------------------------------------
' Calculate Pi to 14 decimal places which should be
' 3.14159265358979
' ------------------------------------------------------------
  Pi = Atn(1) * 4
  
' ------------------------------------------------------------
' Use API call to get volume serial number of drive C:.
' Assumes there will always be a drive C: since we are using
' Windows.
' ------------------------------------------------------------
  lngRetVal = GetVolumeInformation("C:\", "", 0, lngVolSn, 0, 0, "", 0)
   
' ------------------------------------------------------------
' get number of milliseconds since the last reboot
' ------------------------------------------------------------
  lngTickCnt = GetTickCount
  
' ------------------------------------------------------------
' Pre-Initialize the VB random generator
' ------------------------------------------------------------
  varTmpSeed = lngTickCnt + Pi
  Randomize varTmpSeed
  
' ------------------------------------------------------------
' Generate the start and end points for the next loop
' ------------------------------------------------------------
  Do
      lngStart = Month(Now)   ' save the month
      
      ' perform Modulo math.  If the result exceeds the max (12)
      ' then wrap around and start back to the beginning.
      ' Add seven months to the previous value.
      lngStart = (lngStart + 7) Mod 12

      ' add the system seconds to the number of days plus
      ' the number of milliseconds since the last reboot.
      lngTmpValue = lngTickCnt + CLng(Second(Now) + Day(Now))

      ' Perform Modulo math.  If the result exceeds the max (60)
      ' then wrap around and start back at the beginning.
      ' Multiply the calculated value from the previous step
      ' by the calculated month value.
      lngFinish = (lngTmpValue * lngStart) Mod 60

      ' if the initial value is greater than the other then
      ' swap places.  Sometimes the SWAP function is not
      ' always available.  I chose to do this manually.
      If lngStart > lngFinish Then
          lngTmpHold = lngStart       ' move value 1 to holding area
          lngStart = lngFinish        ' move value 2 to value 1
          lngFinish = lngTmpHold      ' move value in holding area to value 2
      End If

  Loop While lngStart >= lngFinish    ' we need a beginning and an end

' ------------------------------------------------------------
' Insure that there are more than 5 iterations in the next
' loop
' ------------------------------------------------------------
  If (lngFinish - lngStart) < 6 Then
      lngFinish = lngFinish + 5
  End If
  
' ------------------------------------------------------------
' Calculate the new seed value in a loop
' ------------------------------------------------------------
  For n = lngStart To lngFinish
  
      ' capture current system date/time stamp and
      ' add it to the new seed value
      varRndSeed = varRndSeed + CDbl(Now)
      
      ' Now we make a unique number
      varRndSeed = Sqr(varRndSeed) + Pi
      varRndSeed = CDbl((Timer + varRndSeed) * sngMultiplier)
      varRndSeed = varRndSeed + lngTickCnt + lngVolSn + m_dblHandleTotal
      varRndSeed = (varRndSeed / (n + 1))
      
      ' if the looping index is divided equally by three
      ' then perform some calculation changes in one
      ' direction else go in another direction.  Helps
      ' us from falling into a pattern.
      If n Mod 3 = 0 Then
          sngMultiplier = sngMultiplier * 0.5
          varRndSeed = varRndSeed * Pi
      Else
          sngMultiplier = sngMultiplier + 0.5
          varRndSeed = varRndSeed * sngMultiplier
      End If
  
      varDecimal = Mid(varRndSeed, InStr(varRndSeed, ".")) ' save just decimal value
      varDecimal = CDec((varDecimal + Pi) * (Rnd * 1))
      varRndSeed = varRndSeed + varDecimal
  Next

' ------------------------------------------------------------
' For debugging only.  To turn on debugging, uncomment the
' next line to set bTestMode = True
'
' Sample output:   29595994.47378014152651
'                  45337025.66965503496078
'                 113368419.68224317930845
'                  27484260.41284549482284
'                  42173945.8184387370566633
'                 139529402.7532063705843
' ------------------------------------------------------------
  'bTestMode = True
  If bTestMode Then
      Debug.Print varRndSeed    ' Display the output
      Stop                    ' press F5 to continue
      GoTo Top_Of_Test_Loop   ' Start over
  End If
  
' ------------------------------------------------------------
' Initialize the random generator using the new calculated
' value (64-bit value) as the seed.  Randomize returns a
' data type that stores a 32-bit(4-byte) single-precision
' floating-point number.
' ------------------------------------------------------------
  Randomize varRndSeed

' ------------------------------------------------------------
' Empty variables for security reasons
' ------------------------------------------------------------
  Erase m_dblParentHandle()    ' Remove all the data from the array
  ReDim m_dblParentHandle(0)   ' Resize array so length is not known
  dblCurrTime = 0              ' reset other variables to zero
  dblTmpValue = 0
  m_dblHandleTotal = 0
  lngStart = 0
  lngFinish = 0
  lngTmpValue = 0
  lngTmpHold = 0
  lngVolSn = 0
  lngTickCnt = 0
  lngParam = 0
  sngMultiplier = 0
  varRndSeed = Empty
  varTmpSeed = Empty
  varDecimal = Empty

End Sub

Public Sub Seed3_Random_Generator()

' ***************************************************************************
' Routine:       Seed3_Random_Generator
'
' Description:   Generates a 64-bit (8-byte word) double precision number
'                as the new seed for the random generator.  The random
'                generator takes this number and then creates a 32-bit (4-
'                byte) single precision number.  Reseed as often as you
'                want.
'
'                This type of random generation is needed for gaming,
'                password creation, encryption, etc.  A calculated value
'                that cannot be easily compromised.
'
' Parameters:
'
' Return Values:
'
' Special Logic:
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 24-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ------------------------------------------------------------
' Define local variables
' ------------------------------------------------------------
  Const iCounter As Integer = 10
  
  Dim varRndSeed As Variant
  Dim i As Integer
  Dim lngLowerLimit As Long
  Dim lngUpperLimit As Long
  Dim bTestMode As Boolean
  
Top_Of_Test_Loop:
' ------------------------------------------------------------
' Initialize the VB random number generator seed
' ------------------------------------------------------------
  bTestMode = False
  Randomize (Timer + CDbl(Now))
  lngLowerLimit = 0
  lngUpperLimit = 0
  varRndSeed = Empty
  
' ------------------------------------------------------------
' Initialize the lower and upper limits
' ------------------------------------------------------------
  While lngLowerLimit >= lngUpperLimit
      lngLowerLimit = (Rnd * 8192)           '    0 to 8192
      lngUpperLimit = (Rnd * 24574) + 8193   ' 8193 to 32767
  Wend
  
' ------------------------------------------------------------
' Initialize the new seed
' ------------------------------------------------------------
  While varRndSeed = 0
      varRndSeed = ((lngUpperLimit - lngLowerLimit + 1) * Rnd)
      varRndSeed = (varRndSeed + lngLowerLimit) + CDbl(Now)
  Wend
 
' ------------------------------------------------------------
' Loop thru and recalculate the value of the new seed
' ------------------------------------------------------------
  For i = 0 To iCounter
      varRndSeed = (lngUpperLimit / lngLowerLimit) + (varRndSeed / 25) * lngLowerLimit
      varRndSeed = Mid(varRndSeed, InStr(varRndSeed, "."))  ' save decimal value only
      varRndSeed = CDec(varRndSeed)
  Next

' ------------------------------------------------------------
' For debugging only.  To turn on debugging, uncomment the
' next line to set bTestMode = True
'
' Sample Output:    0.585557684654190418663810265
'                   0.0135991812904
'                   0.990122159816596628591411226
'                   0.23230047005405142543520764
'                   0.475468645882074791294757949
' ------------------------------------------------------------
  'bTestMode = True
  If bTestMode Then
      Debug.Print varRndSeed    ' display output
      Stop                    ' press F5 to continue
      GoTo Top_Of_Test_Loop   ' Start over
  End If
  
' ------------------------------------------------------------
' Initialize the random generator using the new calculated
' value (64-bit value) as the seed.  Randomize returns a
' data type that stores a 32-bit(4-byte) single-precision
' floating-point number.
' ------------------------------------------------------------
  Randomize varRndSeed
 
' ------------------------------------------------------------
' Empty the variables for security reasons
' ------------------------------------------------------------
  varRndSeed = Empty
  lngUpperLimit = 0
  lngLowerLimit = 0
  
End Sub

Sub Test()

' ----------------------------------------------------------
' TEST STRING DATA (Building true random data)
'
' Written by Kenneth Ives                    kenaso@home.com
'
' Rename this routine to "Main" and then press F5 to execute.
' Look at the "Debug Immediate Window" to see the results.
' To change the seeding routine, go into the
' Build_Random_Data routine and uncomment the appropriate
' seed line.  To display the seed values, go into their
' routine.  Uncomment the selected lines.
' ----------------------------------------------------------

' ------------------------------------------------------------
' Define variables
' ------------------------------------------------------------
  Dim strTestData As String
  Dim lngTestLen As Long
  
' ------------------------------------------------------------
' Initialize variables
' ------------------------------------------------------------
  strTestData = ""
  lngTestLen = 1024  ' 64000
  Screen.MousePointer = vbHourglass
  
' ------------------------------------------------------------
' Create the random data string.  See Build_Random_Data
' procedure for an explanation of the True/False parameters.
' Change these parameters and see how your output varies.
' ------------------------------------------------------------
  strTestData = Build_Random_Data(lngTestLen, False, False)
  Debug.Print "All 255 ASCII characters, decimal value display"
  Debug.Print strTestData
  Debug.Print "-----------  Separator  ---------- "
  
  strTestData = Build_Random_Data(lngTestLen, True, False)
  Debug.Print "All 255 ASCII characters, 2-Character hex display"
  Debug.Print strTestData
  Debug.Print "-----------  Separator  ---------- "
  
  strTestData = Build_Random_Data(lngTestLen, False, True)
  Debug.Print "Printable keyboard characters only (33-126), decimal value display"
  Debug.Print strTestData
  Debug.Print "-----------  Separator  ---------- "
  
  strTestData = Build_Random_Data(lngTestLen, True, True)
  Debug.Print "Printable keyboard characters only (33-126), 2-Character hex display"
  Debug.Print strTestData
  Debug.Print "==========  End Of Test =========="
  
  Screen.MousePointer = vbNormal
  End

End Sub

