Attribute VB_Name = "Module7"
' COPY THIS BLOCK TO THE MODULE (GENERAL) (DECLARATIONS)
Public Function PrintBlock(x As Single, y As Single, Text As String, BlockWidth As Single, justify As String) As Single

' Outputs a block of left, centre, right, full or force justified text to the printer.
' RETURN VALUES - IF CARRIED OUT OK then Height of Block is returned
'                 IF AN ERROR then a negative value (-1) is returned
PrintBlock = -1 ' start off at the error value
If BlockWidth = 0 Then GoTo parameterfault
justify = LCase(justify)
' default to "left" if the justify string is not correct
If justify <> "right" And justify <> "centre" And justify <> "force" And justify <> "full" Then
  justify = "left"
End If
On Error GoTo pjerror
Dim pointer As Long, Counter As Long
Dim nextspace As Long
Dim parafinished As Long, wordlength As Long
Dim tempword As String, space As Long
Dim maxwords As Long: maxwords = 10000
ReDim words(maxwords) As String
Dim paratext As String, extraspace As Single
Dim parapointer As Long, oldpointer As Long
Dim spacewidth As Single, wordwidth As Single, LineWidth As Single
Dim linefirstword As Long, linelastword As Long, n As Long
Dim lastline As Boolean
' This first simple routine will deal with paragraphs up to
' [maxwords] words. Later, increase this to any number of
' words by temporarily stopping and outputting when the
' number of words found exceeds a certain amount, and then
' re-starting at the new position in the text. (Note: The
' routine can sort out 100 words in less than 5 milliseconds).
If Text = "" Then PrintBlock = 0: Exit Function
BlockWidth = Abs(BlockWidth)
Printer.CurrentY = y
spacewidth = Printer.TextWidth(" ")
' Find the first paragraph
parapointer = 1
moretext:
oldpointer = parapointer
parapointer = InStr(parapointer, Text, Chr$(13))
If parapointer = 0 Then
  paratext = Mid$(Text, oldpointer)
  GoSub paragraph
  GoTo textfinished
Else
  ' Note: although it says in the Help Files that the length
  ' specified in Mid$ must be 1 or greater, it works fine
  ' (and returns an empty string) if the length specified
  ' is zero - which it will be whenever the first byte
  ' of the tested section is chr$(13) and therefore
  ' parapointer is equal to oldpointer.
  paratext = Mid$(Text, oldpointer, parapointer - oldpointer)
  GoSub paragraph
  parapointer = parapointer + 2 ' skip chr$(13) and chr$(10)
  If parapointer > Len(Text) Then
    GoTo textfinished
  Else
    GoTo moretext
  End If
End If
textfinished:
PrintBlock = Printer.CurrentY - y
' PrintBlock returns the height of the printed block (-1 if error)
Exit Function
paragraph:
pointer = 1: Counter = 1: parafinished = False
If paratext = "" Then GoTo gotwords ' empty = new para
While Mid$(paratext, pointer, 1) = " "
pointer = pointer + 1
Wend
If pointer > Len(paratext) Then
  paratext = "" ' empty = new para
  GoTo gotwords
End If
' pointer now points to the first non-space character
' so find the next space AFTER that character
wordloop:
nextspace = InStr(pointer, paratext, " ")
If nextspace > 0 Then ' there is another space
  ' The word is the part before the space just found
  words(Counter) = Mid$(paratext, pointer, nextspace - pointer)
Else
  ' The word is the remaining part of the paragraph
  words(Counter) = Mid$(paratext, pointer)
  parafinished = True
End If
If parafinished = True Then GoTo gotwords
' now move the pointer to the next non-space character
pointer = nextspace + 1 ' bypass the space just found
While Mid$(paratext, pointer, 1) = " " ' and look for others
pointer = pointer + 1
Wend
' Temporary word limit due to size of word array
If Counter >= maxwords Then GoTo gotwords
' Finished if past the end of the paragraph
If pointer > Len(paratext) Then GoTo gotwords
' Finished if the last word has been dealt with
If nextspace = 0 Then GoTo gotwords
Counter = Counter + 1
GoTo wordloop
gotwords:
GoSub makelines
Return
'
makelines:
lastline = False
If paratext = "" Then
  Printer.Print "" ' Move to start of new para
  GoTo endmakelines
End If
linefirstword = 1
firstwordofline:
linelastword = linefirstword: LineWidth = 0
wordwidth = Printer.TextWidth(words(linelastword))
LineWidth = LineWidth + wordwidth
If linelastword >= Counter Then
  lastline = True
  GoSub printerline
  GoTo endmakelines
End If
nextwordofline:
wordwidth = spacewidth + Printer.TextWidth(words(linelastword + 1))
If (LineWidth + wordwidth) <= BlockWidth Then
  LineWidth = LineWidth + wordwidth
  linelastword = linelastword + 1
  If linelastword < Counter Then
  GoTo nextwordofline
  Else
  lastline = True
  GoSub printerline
  GoTo endmakelines
  End If
Else
  GoSub printerline
  linefirstword = linelastword + 1
  GoTo firstwordofline
End If
endmakelines:
Return
'
printerline:
If justify = "force" Or justify = "full" Then
  Printer.CurrentX = x
  If linefirstword = linelastword Then
    Printer.Print words(linefirstword)
  Else
    extraspace = linelastword - linefirstword
    extraspace = (BlockWidth - LineWidth) / extraspace
    If justify = "full" And lastline = True Then extraspace = 0
    For n = linefirstword To linelastword
    Printer.Print words(n);
    If n = (linelastword - 1) And lastline = False Then
      ' right justify the last word to prevent slightly "jagged"
      ' paragraph edges (possibly due to accumulated 300 dpi
      ' rounding errors.
      Printer.CurrentX = (x + BlockWidth) - Printer.TextWidth(words(n + 1))
    Else
      Printer.CurrentX = Printer.CurrentX + spacewidth + extraspace
    End If
    Next n
    Printer.Print
  End If
  Return
End If
If justify = "left" Then
  Printer.CurrentX = x
  For n = linefirstword To linelastword
  Printer.Print words(n);
  Printer.CurrentX = Printer.CurrentX + spacewidth
  Next n
  Printer.Print
  Return
End If
If justify = "right" Then
  Printer.CurrentX = x + (BlockWidth - LineWidth)
  For n = linefirstword To linelastword
  Printer.Print words(n);
  Printer.CurrentX = Printer.CurrentX + spacewidth
  Next n
  Printer.Print
  Return
End If
If justify = "centre" Then
  Printer.CurrentX = x + (BlockWidth - LineWidth) / 2
  For n = linefirstword To linelastword
  Printer.Print words(n);
  Printer.CurrentX = Printer.CurrentX + spacewidth
  Next n
  Printer.Print
  Return
End If
Return
'
pjerror:
Resume printingfault
'
printingfault:
PrintBlock = -1
On Error GoTo 0
Exit Function
parameterfault:
PrintBlock = -1
Exit Function
End Function


